<?php
require_once(dirname(__FILE__).'/vendor/autoload.php');
use at\externet\eps_bank_transfer;

/* Eps Payment Gateway Class */
class WC_Eps extends WC_Payment_Gateway 
{
        // Setup our Gateway's id, description and other values
    function __construct()
    {
        // The global ID for this Payment method
        $this->id = "eps";

        // The Title shown on the top of the Payment Gateways Page next to all the other Payment Gateways
        $this->method_title = __( "Eps", 'eps' );

        // The description for this Payment Gateway, shown on the actual Payment options page on the backend
        $this->method_description = __( "Eps Payment Gateway Plug-in for WooCommerce", 'eps' );

        // The title to be used for the vertical tabs that can be ordered top to bottom
        $this->title = __( "Eps", 'Eps' );

        // If you want to show an image next to the gateway's name on the frontend, enter a URL to an image.
        $this->icon = null; 

        $this->has_fields = false; 

        // This basically defines your settings which are then loaded with init_settings()
        $this->init_form_fields();

        // After init_settings() is called, you can get the settings and load them into variables, e.g:
        // $this->title = $this->get_option( 'title' );
        $this->init_settings();
        
        // Turn these settings into variables we can use
        foreach ( $this->settings as $setting_key => $value ) 
        {
            $this->$setting_key = $value;
        }

        if(get_locale()=="de_DE")
        {
            $lang = $this->get_de_translation();
        }
        else
        {
            $lang = $this->get_en_translation();
        }
   
        // Define user set variables.
        $this->title         = $lang['TITLE'];
        $this->description   = $lang['DESCRIPTION'];
        
        $this->instructions  = $this->get_option( 'instructions' );
        $this->test_mode     = $this->get_option( 'test_mode' );
        $this->user_id       = $this->get_option( 'user_id' );
        $this->pin           = $this->get_option( 'pin' );
        $this->bic           = $this->get_option( 'bic' );
        $this->iban          = $this->get_option( 'iban' );
      
        // Lets check for SSL
        add_action( 'admin_notices', array( $this,  'do_ssl_check' ) );     
        
        if ( version_compare( WOOCOMMERCE_VERSION, '2.0.1', '>=' ) ) {
                add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( &$this, 'process_admin_options' ) );
         } else {
            add_action( 'woocommerce_update_options_payment_gateways', array( &$this, 'process_admin_options' ) );
        }

        add_action('woocommerce_receipt_eps', array(&$this, 'receipt_page'));
              
    } 


    function init_form_fields()
    {
        if(get_locale()=="de_DE")
        {
            $lang = $this->get_de_translation();
        }
        else
        {
            $lang = $this->get_en_translation();
        }
        
        /* Admin fields */
        $this->form_fields = array(
            // 'enabled' => array(
            //     'title' => __( $lang['FIELD_ENABLE'], 'wcwcCpg1' ),
            //     'type' => 'checkbox',
            //     'label' => __( $lang['FIELD_ENABLE_LABEL'], 'wcwcCpg1' ),
            //     'default' => 'no'
            // ),
              'test_mode' => array(
                'title' => __( $lang['FIELD_TESTMODE'], 'wcwcCpg1' ),
                'type' => 'checkbox',
                'label' => __( $lang['FIELD_TESTMODE_LABEL'], 'wcwcCpg1' ),
                'default' => 'yes'
            ),
            
             'user_id' => array(
                'title' => __( 'User-ID', 'wcwcCpg1' ),
                'type' => 'text',
                'description' => __( '', 'wcwcCpg1' ),
                'desc_tip' => true,
                'default' => __( '', 'wcwcCpg1' )
            ),
            'pin' => array(
                'title' => __( 'PIN', 'wcwcCpg1' ),
                'type' => 'text',
                'description' => __( '', 'wcwcCpg1' ),
                'desc_tip' => true,
                'default' => __( '', 'wcwcCpg1' )
            ),
            'bic' => array(
                'title' => __( 'BIC', 'wcwcCpg1' ),
                'type' => 'text',
                'description' => __( '', 'wcwcCpg1' ),
                'desc_tip' => true,
                'default' => __( '', 'wcwcCpg1' )
            ),
            'iban' => array(
                'title' => __( 'IBAN', 'wcwcCpg1' ),
                'type' => 'text',
                'description' => __( '', 'wcwcCpg1' ),
                'desc_tip' => true,
                'default' => __( '', 'wcwcCpg1' )
            ),
        );
    }

    /* requireid function */
    public function admin_options(){
        echo '<h3>'.__('Eps Payment Gateway', 'leftor').'</h3>';
        echo '<table class="form-table">';
        // Generate the HTML For the settings form.
        $this -> generate_settings_html();
        echo '</table>';

    }

    
    // Check if we are forcing SSL on checkout pages
    // Custom function not required by the Gateway
    // requireid
    public function do_ssl_check() {
        if( $this->enabled == "yes" ) {
            if( get_option( 'woocommerce_force_ssl_checkout' ) == "no" ) {
                echo "<div class=\"error\"><p>". sprintf( __( "<strong>%s</strong> is enabled and WooCommerce is not forcing the SSL certificate on your checkout page. Please ensure that you have a valid SSL certificate and that you are <a href=\"%s\">forcing the checkout pages to be secured.</a>" ), $this->method_title, admin_url( 'admin.php?page=wc-settings&tab=checkout' ) ) ."</p></div>";  
            }
        }       
    }

    /**
     * Process the payment and return the result
     * requireid 
     **/
    function process_payment($order_id){
        global $woocommerce;

        $order = new WC_Order($order_id);

        $validation = $this->validate_form_fields($order);

        if($validation["response"]=="false"){
            wc_add_notice($validation["message"], 'error');
            return;
        }            
        
        return array(
        	'result' => 'success', 
        	'redirect' => add_query_arg(
        		'order',
            	$order->get_id(), add_query_arg('key', $order->get_order_key(), wc_get_checkout_url())
            	)
             );
    }


    /**
     * Validate WooCommerce Form fields
     **/
    function validate_form_fields($order){

        if($this->form_language == "en"){
            $lang = $this->get_en_translation();
        }
        elseif($this->form_language == "de"){
            $lang = $this->get_de_translation();
        }

        if(strlen($order->get_billing_first_name()) < 3 || strlen($order->get_billing_first_name()) > 11){
            $validation['response'] = "false";
            $validation['message'] = $lang['FIRST_NAME_ERROR'];
            return $validation;
        }
        if(strlen($order->get_billing_last_name()) < 3 || strlen($order->get_billing_last_name()) > 18){
            $validation['response'] = "false";
            $validation['message'] = $lang['LAST_NAME_ERROR'];
            return $validation;
        }
        if(strlen($order->get_billing_address_1()) < 3 ||strlen($order->get_billing_address_1()) > 300){
            $validation['response'] = "false";
            $validation['message'] = $lang['ADDRESS_ERROR'];
            return $validation;
        }
        if(strlen($order->get_billing_city()) < 3 || strlen($order->get_billing_city()) > 30){
            $validation['response'] = "false";
            $validation['message'] = $lang['CITY_ERROR'];
            return $validation;
        }
        if(strlen($order->get_billing_postcode()) < 4 || strlen($order->get_billing_postcode()) > 9){
            $validation['response'] = "false";
            $validation['message'] = $lang['ZIP_ERROR'];
            return $validation;
        }
   /*     if(strlen($order->get_billing_phone()) < 3 || strlen($order->get_billing_phone()) > 30){
            $validation['response'] = "false";
            $validation['message'] = $lang['PHONE_ERROR'];
            return $validation;
        } */
        if(strlen($order->get_billing_email()) < 7 || strlen($order->get_billing_email()) > 100){
            $validation['response'] = "false";
            $validation['message'] = $lang['EMAIL_ERROR'];
            return $validation;
        }

    }


    function receipt_page($order)
    {   
        if(get_locale()=="de_DE")
        {
            $lang = $this->get_de_translation();
        }
        else
        {
            $lang = $this->get_en_translation();
        }
        
        echo '<p>'.__($lang['RECIEPT_PAGE'], 'leftor').'</p>';
        
        echo $this->redirect_data($order);           
    }

     /**
     * Redirect function EPS
     **/    
    public function redirect_data($order_id)
    {           
        global $woocommerce;

        /* Order data */
        $order       = new WC_Order( $order_id );  
        $order_info  = $order_id.'_'.date("dmy");
        $order_data  = $order->get_data();
        $currency    = $order_data["currency"];
        $amount      = $order->order_total;
        $order_total = filter_var($amount, FILTER_SANITIZE_NUMBER_INT);
        $name        = get_bloginfo('name');
        $items       = $order->get_items();
        
        //$full_name   = $order->billing_first_name ." " .$order->billing_last_name;
        /* Admin data */
        $user_id = $this->user_id;
        $pin     = $this->pin;
        $bic     = $this->bic;
        $iban    = $this->iban;
        $test    = $this->test_mode;
        
    

        $site_url     = site_url();
        $responseUrl  = $site_url."/wp-content/plugins/woocommerce-eps/confirm.php";
        $successUrl   = $site_url."/wp-content/plugins/woocommerce-eps/ok.php?order_id=$order_id";
        $failUrl      = $site_url."/wp-content/plugins/woocommerce-eps/fail.php?order_id=$order_id";

        if($test == "yes")
        {
            $targetUrl = 'https://routing.eps.or.at/appl/epsSO-test/transinit/eps/v2_5';
        }
        else 
        {
            $targetUrl = 'https://routing.eps.or.at/appl/epsSO/transinit/eps/v2_5';
        }
 

        $transferMsgDetails = new eps_bank_transfer\TransferMsgDetails(
            $responseUrl,
            $successUrl,
            $failUrl
        );
    
        $transferInitiatorDetails = new eps_bank_transfer\TransferInitiatorDetails(
            $user_id,
            $pin,
            $bic,
            $name,
            $iban,
            $order_id,
            intval($order_total),
            $transferMsgDetails
        );
        $transferInitiatorDetails->RemittanceIdentifier = $order_id;
        $transferInitiatorDetails->SetExpirationMinutes(60);
        
     
        foreach($items as $item) 
        {
            $article = new eps_bank_transfer\WebshopArticle
            (
                $item->get_name(),
                $item->get_quantity(),
                intval($item->get_subtotal())
            );

            $transferInitiatorDetails->WebshopArticles[] = $article;
        }


        // Send TransferInitiatorDetails to Scheme Operator
        $soCommunicator = new eps_bank_transfer\SoCommunicator();
        $plain = $soCommunicator->SendTransferInitiatorDetails($transferInitiatorDetails, $targetUrl);
     
        $xml = new \SimpleXMLElement($plain);
        $soAnswer = $xml->children(eps_bank_transfer\XMLNS_epsp);

        $errorDetails = $soAnswer->BankResponseDetails->ErrorDetails;
        
        if (('' . $errorDetails->ErrorCode) != '000')
        {
            $errorCode = '' . $errorDetails->ErrorCode;
            $errorMsg = '' . $errorDetails->ErrorMsg;
            echo "<h3>Error Code: ".$errorCode."<br></h3>";
            echo "<h3>Error Message: ".$errorMsg."<br></h3>";
            $order_err = new WC_Order($order_id);
            $order_err->update_status('cancelled', 'order_note');
        }

        else
        {
            $redirectUrl = $soAnswer->BankResponseDetails->ClientRedirectUrl->__toString();
            wp_redirect($redirectUrl);
        }

        ///END  
    }

     /*
    ------------------
    Language: English
    ------------------
    */
    public function get_en_translation()
    {
    
        $lang = array();
         
        $lang['TITLE'] = "EPS Payment";
        $lang['DESCRIPTION'] = 'EPS Secure Payment';
        $lang['FIELD_ENABLE'] = 'EPS Payment module Enable/Disable';
        $lang['FIELD_ENABLE_LABEL'] = 'Enable EPS module';
        $lang['FIELD_TESTMODE'] = 'Test mode - Enable/Disable';
        $lang['FIELD_TESTMODE_LABEL'] = 'Enable EPS Test mode';
     
       $lang['RECIEPT_PAGE'] = 'Redirecting...';

        //Thankyou page messages
        $lang['THANKYOU_SUCCESS'] = 'Thank you for shopping with us. Your account has been charged and your transaction is successful. We will be shipping your order to you soon.';
        $lang['orange_SUCCESS'] = 'orange payment successful<br/>Unique Id from orange: ';
        $lang['THANKYOU_PENDING'] = 'Thank you for shopping with us. Right now your payment status is pending, We will keep you posted regarding the status of your order through e-mail';
        $lang['orange_PENDING'] = 'orange payment status is pending<br/>Unnique Id from orange: ';
        $lang['SECURITY_ERROR'] = 'Security Error. Illegal access detected';
        $lang['THANKYOU_DECLINED'] = 'Thank you for shopping with us. However, the transaction has been declined.';
        $lang['THANKYOU_DECLINED_NOTE'] = 'Transaction Declined: ';

        //Payment notes
        $lang['PAYMENT_COMPLETED'] = 'Eps payment completed.';
        $lang['TRANSACTION_FAILED'] = 'Transaction failed.';

        $lang['INSTALLMENT_PAYMENT_DISABLED'] = 'Paying in installments not enabled!';
        $lang['INSTALLMENT_MINIMUM_REQUIREMENT'] = 'Minimum required order cost is not met!';

        return $lang;

    }
    
    /*
    ------------------
    Language: Deutsch
    ------------------
    */
    public function get_de_translation()
    {
    
        $lang = array();
         
        $lang['TITLE'] = 'eps-Überweisung';
        $lang['DESCRIPTION'] = 'eps-Überweisung ist ein sicheres Online-Bezahlsystem. Es wurde in Kooperation mit den österreichischen Banken entwickelt, um Online-Einkäufe über das sichere Online-Banking abwickeln zu können.“';
        $lang['FIELD_ENABLE'] = 'EPS-Bezahlmodul - Aktivieren/Deaktivieren';
        $lang['FIELD_ENABLE_LABEL'] = 'EPS Modul aktivieren';
        $lang['FIELD_TESTMODE'] = 'Testen - Aktivieren/Deaktivieren';
        $lang['FIELD_TESTMODE_LABEL'] = 'Aktivieren EPS Testen';
        
        $lang['RECIEPT_PAGE'] = 'Redirecting...';

        //Thankyou page messages
        $lang['THANKYOU_SUCCESS'] = 'Thank you for shopping with us. Your account has been charged and your transaction is successful. We will be shipping your order to you soon.';
        
        $lang['THANKYOU_PENDING'] = 'Thank you for shopping with us. Right now your payment status is pending, We will keep you posted regarding the status of your order through e-mail';
      
        $lang['SECURITY_ERROR'] = 'Security Error. Illegal access detected';
        $lang['THANKYOU_DECLINED'] = 'Thank you for shopping with us. However, the transaction has been declined.';
        $lang['THANKYOU_DECLINED_NOTE'] = 'Transaction Declined: ';

        //Payment notes
        $lang['PAYMENT_COMPLETED'] = 'Eps payment completed.';
        $lang['TRANSACTION_FAILED'] = 'Transaction failed.';

        $lang['INSTALLMENT_PAYMENT_DISABLED'] = 'Paying in installments not enabled!';
        $lang['INSTALLMENT_MINIMUM_REQUIREMENT'] = 'Minimum required order cost is not met!';

        return $lang;
    }
}

?>